@@ -3,12 +3,13 @@
 
   SPDX-FileCopyrightText: 2008-2015 Nick Shaforostoff <shafff@ukr.net>
   SPDX-FileCopyrightText: 2018-2019 Simon Depiets <sdepiets@gmail.com>
+  SPDX-FileCopyrightText: 2025 Finley Watson <fin-w@tutanota.com>
 
   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 */
 
 #include "lokalizemainwindow.h"
-#include "actionproxy.h"
+#include "config-lokalize.h"
 #include "editortab.h"
 #include "filesearchtab.h"
 #include "jobs.h"
@@ -20,6 +21,7 @@
 #include "projectmodel.h"
 #include "projecttab.h"
 #include "resizewatcher.h"
+#include "statusbar.h"
 #include "tmtab.h"
 #include "tools/widgettextcaptureconfig.h"
 
@@ -65,6 +67,7 @@
 #include <QStringLiteral>
 #include <QTabBar>
 #include <QTabWidget>
+#include <QtAssert>
 #include <QtContainerFwd>
 #include <QtLogging>
 
@@ -74,7 +77,9 @@ LokalizeMainWindow::LokalizeMainWindow()
     , m_welcomePage(new QWidget(this))
     , m_editorActions(new QActionGroup(this))
     , m_managerActions(new QActionGroup(this))
+    , m_statusBar(new LokalizeStatusBar(this))
 {
+    setStatusBar(m_statusBar);
     // DocumentMode removes the unpleasant borders around the tab pages.
     m_mainTabs->setDocumentMode(true);
     m_mainTabs->setMovable(true);
@@ -147,11 +152,6 @@ LokalizeMainWindow::LokalizeMainWindow()
     connect(Project::instance(), &Project::configChanged, this, &LokalizeMainWindow::projectSettingsChanged);
     connect(Project::instance(), &Project::closed, this, &LokalizeMainWindow::queryAndCloseProject);
 
-    for (int i = ID_STATUS_CURRENT; i <= ID_STATUS_ISFUZZY; i++) {
-        m_statusBarLabels.append(new QLabel());
-        statusBar()->insertWidget(i, m_statusBarLabels.last(), 2);
-    }
-
     setAttribute(Qt::WA_DeleteOnClose, true);
 
     if (!qApp->isSessionRestored()) {
@@ -160,7 +160,9 @@ LokalizeMainWindow::LokalizeMainWindow()
         readProperties(stateGroup);
     }
 
+#if HAVE_DBUS
     registerDBusAdaptor();
+#endif
 
     QTimer::singleShot(0, this, &LokalizeMainWindow::initLater);
 }
@@ -221,11 +223,13 @@ bool LokalizeMainWindow::eventFilter(QObject *object, QEvent *event)
 void LokalizeMainWindow::showTabs()
 {
     m_welcomePageAndTabsPage->setCurrentIndex(1);
+    statusBar()->show();
 }
 
 void LokalizeMainWindow::showWelcome()
 {
     m_welcomePageAndTabsPage->setCurrentIndex(0);
+    statusBar()->hide();
 }
 
 void LokalizeMainWindow::activateTabByPageWidget(QWidget *w)
@@ -240,26 +244,42 @@ void LokalizeMainWindow::activateTabAtIndex(int i)
         return;
     else
         showTabs();
+
     int indexPriorToSwitching = m_mainTabs->currentIndex();
     m_mainTabs->setCurrentIndex(i);
+    m_statusBar->clear();
+    m_statusBar->disconnectSignals();
     previousActiveTabIndex = indexPriorToSwitching;
+
     if (m_projectTab && m_mainTabs->indexOf(m_projectTab) == i) {
-        m_projectTab->statusBarItems.registerStatusBar(statusBar(), m_statusBarLabels);
+        m_statusBar->connectSignals(m_projectTab);
+        m_projectTab->updateStatusBarContents();
     } else if (m_translationMemoryTab && m_mainTabs->indexOf(m_translationMemoryTab) == i) {
-        m_translationMemoryTab->statusBarItems.registerStatusBar(statusBar(), m_statusBarLabels);
+        m_statusBar->connectSignals(m_translationMemoryTab);
+        m_translationMemoryTab->updateStatusBarContents();
+    } else if (m_fileSearchTab && m_mainTabs->indexOf(m_fileSearchTab) == i) {
+        m_statusBar->connectSignals(m_fileSearchTab);
+        m_fileSearchTab->updateStatusBarContents();
     } else if (EditorTab *editorTab = qobject_cast<EditorTab *>(m_mainTabs->currentWidget())) {
         if (m_editorStateOfAllOpenEditors.isEmpty()) {
             m_editorStateOfAllOpenEditors = editorTab->state().qMainWindowState;
         } else {
             editorTab->restoreState(m_editorStateOfAllOpenEditors);
         }
+        m_statusBar->connectSignals(editorTab);
         editorTab->setProperFocus();
-        editorTab->statusBarItems.registerStatusBar(statusBar(), m_statusBarLabels);
+        editorTab->updateStatusBarContents();
     }
+
     // This disconnects the old keyboard shortcuts and connects those
     // related to the currently visible tab.
     guiFactory()->removeClient(m_activeTabPageKeyboardShortcuts);
-    m_activeTabPageKeyboardShortcuts = static_cast<LokalizeTabPageBase *>(m_mainTabs->currentWidget())->guiClient();
+    QWidget *currentTab = m_mainTabs->currentWidget();
+    if (LokalizeTabPageBase *activeTab = qobject_cast<LokalizeTabPageBase *>(currentTab)) {
+        m_activeTabPageKeyboardShortcuts = activeTab->guiClient();
+    } else if (LokalizeTabPageBaseNoQMainWindow *activeTab = qobject_cast<LokalizeTabPageBaseNoQMainWindow *>(currentTab)) {
+        m_activeTabPageKeyboardShortcuts = activeTab->guiClient();
+    }
     guiFactory()->addClient(m_activeTabPageKeyboardShortcuts);
 }
 
@@ -352,8 +372,8 @@ EditorTab *LokalizeMainWindow::fileOpen(QString filePath, int entry, bool setAsA
 
     // Set suggestedDirPath to file path of current tab.
     QString suggestedDirPath;
-    if (QWidget *activeTab = qobject_cast<LokalizeSubwindowBase *>(m_mainTabs->currentWidget())) {
-        QString currentlyActiveTabRelatedFilePath = static_cast<LokalizeSubwindowBase *>(activeTab)->currentFilePath();
+    if (QWidget *activeTab = qobject_cast<LokalizeTabPageBase *>(m_mainTabs->currentWidget())) {
+        QString currentlyActiveTabRelatedFilePath = static_cast<LokalizeTabPageBase *>(activeTab)->currentFilePath();
         if (!currentlyActiveTabRelatedFilePath.isEmpty())
             suggestedDirPath = QFileInfo(currentlyActiveTabRelatedFilePath).absolutePath();
     }
@@ -563,10 +583,12 @@ void LokalizeMainWindow::setupActions()
     actionCategory = file;
 
     // File
-    file->addAction(KStandardAction::Open, this, SLOT(fileOpen()));
+    file->addAction(KStandardActions::Open, this, [this] {
+        fileOpen();
+    });
     m_openRecentFileAction = KStandardAction::openRecent(this, SLOT(fileOpen(QUrl)), ac);
 
-    file->addAction(KStandardAction::Quit, qApp, SLOT(closeAllWindows()));
+    file->addAction(KStandardActions::Quit, qApp, &QApplication::closeAllWindows);
 
     // Settings
     SettingsController *sc = SettingsController::instance();
@@ -600,7 +622,7 @@ void LokalizeMainWindow::setupActions()
     ADD_ACTION_SHORTCUT("tools_tm", i18nc("@action:inmenu", "Translation memory"), Qt::Key_F7)
     connect(action, &QAction::triggered, this, &LokalizeMainWindow::showTM);
 
-    action = tm->addAction(QStringLiteral("tools_tm_manage"), project, SLOT(showTMManager()));
+    action = tm->addAction(QStringLiteral("tools_tm_manage"), project, &Project::showTMManager);
     action->setText(i18nc("@action:inmenu", "Manage translation memories"));
 
     // Project
@@ -608,20 +630,20 @@ void LokalizeMainWindow::setupActions()
     ADD_ACTION_SHORTCUT("project_overview", i18nc("@action:inmenu", "Project overview"), Qt::Key_F4)
     connect(action, &QAction::triggered, this, &LokalizeMainWindow::showProjectOverview);
 
-    action = proj->addAction(QStringLiteral("project_configure"), sc, SLOT(projectConfigure()));
+    action = proj->addAction(QStringLiteral("project_configure"), sc, &SettingsController::projectConfigure);
     action->setText(i18nc("@action:inmenu", "Configure project..."));
 
-    action = proj->addAction(QStringLiteral("project_create"), sc, SLOT(projectCreate()));
+    action = proj->addAction(QStringLiteral("project_create"), sc, &SettingsController::projectCreate);
     action->setText(i18nc("@action:inmenu", "Create software translation project..."));
 
-    action = proj->addAction(QStringLiteral("project_create_odf"), Project::instance(), SLOT(projectOdfCreate()));
+    action = proj->addAction(QStringLiteral("project_create_odf"), Project::instance(), &Project::projectOdfCreate);
     action->setText(i18nc("@action:inmenu", "Create OpenDocument translation project..."));
 
-    action = proj->addAction(QStringLiteral("project_open"), this, SLOT(openProject()));
+    action = proj->addAction(QStringLiteral("project_open"), this, qOverload<>(&LokalizeMainWindow::openProject));
     action->setText(i18nc("@action:inmenu", "Open project..."));
     action->setIcon(QIcon::fromTheme(QStringLiteral("project-open")));
 
-    action = proj->addAction(QStringLiteral("project_close"), this, SLOT(queryAndCloseProject()));
+    action = proj->addAction(QStringLiteral("project_close"), this, &LokalizeMainWindow::closeProject);
     action->setText(i18nc("@action:inmenu", "Close project"));
     action->setIcon(QIcon::fromTheme(QStringLiteral("project-close")));
 
@@ -641,7 +663,7 @@ void LokalizeMainWindow::setupActions()
     ADD_ACTION_SHORTCUT("tools_filesearch_next", i18nc("@action:inmenu", "Find next in files"), Qt::META | Qt::Key_F3)
     connect(action, &QAction::triggered, this, &LokalizeMainWindow::fileSearchNext);
 
-    action = ac->addAction(QStringLiteral("tools_widgettextcapture"), this, SLOT(widgetTextCapture()));
+    action = ac->addAction(QStringLiteral("tools_widgettextcapture"), this, &LokalizeMainWindow::widgetTextCapture);
     action->setText(i18nc("@action:inmenu", "Widget text capture"));
 
 #if KCONFIGWIDGETS_VERSION >= QT_VERSION_CHECK(6, 3, 0)
@@ -797,7 +819,6 @@ void LokalizeMainWindow::projectLoaded()
     if (!projectPath.isEmpty())
         showProjectOverview();
     if (m_projectTab) {
-        m_projectTab->setLegacyUnitsCount(projectStateGroup.readEntry("UnitsCount", 0));
         m_mainTabs->setTabToolTip(m_mainTabs->indexOf(m_projectTab), Project::instance()->path());
     }
     entries = projectStateGroup.readEntry("Entries", entries);
@@ -846,6 +867,7 @@ void LokalizeMainWindow::widgetTextCapture()
 
 // BEGIN DBus interface
 
+#if HAVE_DBUS
 #include "mainwindowadaptor.h"
 
 void LokalizeMainWindow::registerDBusAdaptor()
@@ -853,6 +875,7 @@ void LokalizeMainWindow::registerDBusAdaptor()
     new MainWindowAdaptor(this);
     QDBusConnection::sessionBus().registerObject(QLatin1String("/ThisIsWhatYouWant"), this);
 }
+#endif
 
 int LokalizeMainWindow::lookupInTranslationMemory(DocPosition::Part part, const QString &text)
 {
@@ -1046,7 +1069,13 @@ void LokalizeMainWindow::closeTabAtIndex(int index)
         return;
     }
     // This disconnects the keyboard shortcuts relating to the tab being closed.
-    guiFactory()->removeClient(static_cast<LokalizeTabPageBase *>(m_mainTabs->widget(index)));
+    QWidget *currentTab = m_mainTabs->currentWidget();
+    Q_ASSERT(qobject_cast<LokalizeTabPageBase *>(currentTab) || qobject_cast<LokalizeTabPageBaseNoQMainWindow *>(currentTab));
+    if (LokalizeTabPageBase *tabToDelete = qobject_cast<LokalizeTabPageBase *>(currentTab)) {
+        guiFactory()->removeClient(tabToDelete->guiClient());
+    } else if (LokalizeTabPageBaseNoQMainWindow *tabToDelete = qobject_cast<LokalizeTabPageBaseNoQMainWindow *>(currentTab)) {
+        guiFactory()->removeClient(tabToDelete->guiClient());
+    }
     if (m_mainTabs->currentIndex() == index) {
         m_activeTabPageKeyboardShortcuts = nullptr;
     }
