@@ -13,6 +13,7 @@
 #include "catalog.h"
 #include "headerviewmenu.h"
 #include "lokalize_debug.h"
+#include "prefs_lokalize.h"
 #include "project.h"
 
 #include <KColorScheme>
@@ -118,6 +119,19 @@ QString PoItemDelegate::displayText(const QVariant &value, [[maybe_unused]] cons
     return QStyledItemDelegate::displayText(value, QLocale::system());
 }
 
+ProjectOverviewSortFilterProxyModel::ProjectOverviewSortFilterProxyModel(QObject *parent)
+    : KDirSortFilterProxyModel(parent)
+{
+    connect(Project::instance()->model(), &ProjectModel::totalsChanged, this, &ProjectOverviewSortFilterProxyModel::invalidate);
+    m_hideTranslatedFiles = Settings::hideCompletedItems();
+}
+
+ProjectOverviewSortFilterProxyModel::~ProjectOverviewSortFilterProxyModel()
+{
+    Settings::setHideCompletedItems(m_hideTranslatedFiles);
+    Settings::self()->save();
+}
+
 void ProjectOverviewSortFilterProxyModel::toggleTranslatedFiles()
 {
     m_hideTranslatedFiles = !m_hideTranslatedFiles;
